@@ -11,7 +11,7 @@ tar.gz consisting of the source file and a specfile.
 """
 
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# __COPYRIGHT__
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -33,7 +33,7 @@ tar.gz consisting of the source file and a specfile.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/src/script/src/engine/SCons/Tool/rpm.py none 2019-12-16 19:06:27 none"
+__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 
 import os
 import re
@@ -46,7 +46,7 @@ import SCons.Util
 import SCons.Action
 import SCons.Defaults
 
-def get_cmd(source, env):
+def get_cmd(source, env) -> str:
     tar_file_with_included_specfile = source
     if SCons.Util.is_List(source):
         tar_file_with_included_specfile = source[0]
@@ -110,7 +110,7 @@ RpmBuilder = SCons.Builder.Builder(action = SCons.Action.Action('$RPMCOM', '$RPM
 
 
 
-def generate(env):
+def generate(env) -> None:
     """Add Builders and construction variables for rpm to an Environment."""
     try:
         bld = env['BUILDERS']['Rpm']
